# coding:utf-8
import weakref
import warnings

from qtpy.QtCore import QObject, Signal, QMetaProperty
from qtpy.QtWidgets import QWidget

from .proxy import Proxy


class Ref(Proxy):

    def __init__(self, widget: QWidget, property: str):
        super().__init__()
        self.watchedProperty = property
        self.widgetRef = weakref.ref(widget)
        self._connectSignal(widget)

    @property
    def widget(self):
        return self.widgetRef()

    @property
    def value(self):
        if self.widget is None:
            return None

        return self.widget.property(self.watchedProperty)

    @value.setter
    def value(self, value):
        if value == self.value:
            return

        self.widget.setProperty(self.watchedProperty, value)
        self.valueChanged.emit()

    def _connectSignal(self, widget):
        metaObject = widget.metaObject()
        property = self.watchedProperty
        index = metaObject.indexOfProperty(property)

        if index == -1:
            return

        metaProperty = metaObject.property(index)

        if not metaProperty.hasNotifySignal():
            warnings.warn(f"'{property}' doesn't have NOTIFY signal")
            return

        notifySignal = metaProperty.notifySignal()
        signal = getattr(widget, notifySignal.name().data().decode('utf-8'))
        signal.connect(lambda: self.valueChanged.emit())



# coding: utf-8
import inspect
import weakref

from qtpy.QtCore import QObject
from qtpy.QtWidgets import QWidget

from .ref import Ref
from .proxy import Proxy

class Computed(Proxy):

    def __init__(self, widget: QWidget, property: str, func: callable):
        super().__init__()
        self.func = func
        self.watchedProperty = property
        self.widgetRef = weakref.ref(widget)
        self._register()

    @property
    def widget(self):
        return self.widgetRef()

    @property
    def value(self):
        return self.func()

    def update(self):
        self.widget.setProperty(self.watchedProperty, self.value)

    def _register(self):
        signature = inspect.signature(self.func)
        for arg in signature.parameters.values():
            if isinstance(arg.default, Ref):
                arg.default.valueChanged.connect(self.update)



"""
QMVVM
===================
A Model-View-ViewModel(MVVM) framework for Qt.

Documentation is available in the docstrings and
online at https://qmvvm.org.

Examples are available at https://github.com/zhiyiYo/QMVVM/tree/master/examples.

:copyright: (c) 2025 by zhiyiYo.
:license: Apache v2.0, see LICENSE for more details.
"""

__version__ = "0.0.3"
__author__ = "zhiyiYo"

from .mvvm import QMVVM
from .ref import Ref
from .computed import Computed